package org.bdware.bdledger.api.grpc;

import io.grpc.*;
import io.grpc.stub.AbstractStub;
import org.bdware.bdledger.api.grpc.pb.NodeGrpc;

public class GrpcHeaderInterceptor implements ClientInterceptor {

    private final Metadata extraHeaders;

    public GrpcHeaderInterceptor(Metadata metadata) {
        this.extraHeaders = metadata;
    }

    @Override
    public <ReqT, RespT> ClientCall<ReqT, RespT> interceptCall(MethodDescriptor<ReqT, RespT> method, CallOptions callOptions, Channel next) {
        return new ForwardingClientCall.SimpleForwardingClientCall<ReqT,RespT>(next.newCall(method, callOptions)) {
            @Override
            public void start(Listener<RespT> responseListener, Metadata headers) {
                // 合并额外的头信息
                headers.merge(extraHeaders);
                super.start(responseListener, headers);
            }
        };
    }

    public static <T extends AbstractStub<T>> T attachHeaders(T stub, Metadata metadata) {
        Channel channel = stub.getChannel();
        Channel interceptedChannel = ClientInterceptors.intercept(channel, new GrpcHeaderInterceptor(metadata));
        return stub.withChannel(interceptedChannel);
    }

}
